import com.nttdocomo.util.Timer;
import com.nttdocomo.util.TimerListener;

import java.util.*;
import java.io.*;
import javax.microedition.io.*;
import com.nttdocomo.ui.*;
import com.nttdocomo.io.*;

import input.*;
import drawing.*;
import gaming.*;
/**
 * GameState
 * 
 */
public class GameState implements IGameState{

	private IGamePad vGamePad;
	private VGraphics vg;
	private GameStage stage;
	private int zoomFlag = 0;
	private int frameCnt=0;
	private IGameState startState,dyingState,clearState;
	private PauseMenuState pauseState;

	private int stageCnt=0;
	 public int getStageCnt(){return stageCnt;}
	private boolean giveupFlag=false,pauseFlag=false,replayFlag=false;
	private int giveupCnt=0;
	private String stageBase = "resource:///map/stage";
	
	private String sample=
				"0000000000000000006000000000\n"+
				"0000700000000000006000000000\n"+
				"1111111311111110006000000000\n"+
				"0000000344444444446000070000\n"+
				"0000000300001130001111111311\n"+
				"0000000300001130000000000300\n"+
				"0000080300001130000000780300\n"+
				"1131111100001111111131111111\n"+
				"0030000000000000000030000000\n"+
				"0030000000000080000030000000\n"+
				"1111111113111111111130000000\n"+
				"0000000003000000000030000000\n"+
				"0000000703444444444430007000\n"+
				"0000311111100000000011111113\n"+
				"0000300000000090070000000003\n"+
				"1111111111111111111111111111\n"+
				"";



	CanvasInfo inf;
	boolean ready = false;
    GameState(CanvasInfo ci) {
		inf = ci;
		vg = new VGraphics(ci);
		vGamePad = new VirtualGamePad();
		stageCnt = SystemConfig.stageCnt;
    }


    public IGameState call(){
		ready = false;
		if(startState ==null){
			stage = new GameStage(inf);
			startState = new StageStartState(this,stage,vg,inf);
			dyingState = new PlayerDyingState(this,stage,vg,inf);
			clearState = new StageClearState(this,stage,vg,inf);
			pauseState = new PauseMenuState(this,stage,vg,inf);
		}

		vGamePad.reset();


		int m = stage.getMode();


		if(pauseFlag){
			pauseFlag = false;
			if(pauseState.chgFlag){
				stageCnt = pauseState.stageCnt;
				replayFlag = pauseState.replayFlag;
			}else{
				ready =true;
				return this;
			}
		}


		

		zoomFlag = 0;
		frameCnt = 0;
		giveupFlag= false;
		giveupCnt=0;

		setStage(stageCnt,replayFlag);

		SystemConfig.updateLastStage(stageCnt);
//		SystemConfig.save();
		ready = true;
		return startState.call();
	}

    public IGameState action(){
		if(!ready)return this;
		frameCnt++;
		stage.setGamePad(vGamePad);


		if(stage.action()){
			if(giveupFlag){
				giveupCnt++;
				System.out.println("giveup=" + giveupCnt);
			}
			if(giveupCnt > 100){
				stage.setGiveupFlag();
			}

			if(zoomFlag<0)stage.zoomOut();
			else
			if(zoomFlag>0)stage.zoomIn();
			
			if(pauseFlag)
				return pauseState.call();

		}else{
			int m = stage.getMode();
			if( m == GameStage.MODE_DEAD){
				return dyingState.call();
			}else
			if( m == GameStage.MODE_CLEAR ){
				SystemConfig.saveReplay(stageCnt,stage.getReplayCode(),stage.getFrameCnt());
				if(replayFlag)stageCnt = findNextReplay();
				if(!replayFlag)stageCnt++;
				SystemConfig.updateLastStage(stageCnt);
				return clearState.call();
			}else{

			}
		}

		return this;
	}


	private boolean setStage(int num,boolean replay){//String fileName){
		String dat=loadData(stageBase + num + ".txt");
		if(dat == null){
			if(stageCnt != 0){
				stageCnt = 0;
				return setStage(0,replay);//stageBase + 0 + ".txt");
			}else
				dat = sample;
		}
		stage.setTitle("STAGE " + (stageCnt+1));
		if(replay){
			String tmp = SystemConfig.loadReplay(stageCnt);
			System.out.println("\n\nreplay=\n" + tmp );
			if(!tmp.equals("")){
				dat += "\n\n\n" + tmp;
			}
		}
		stage.loadStage(dat);
		return true;
	}
	
	private String loadData(String fileName){
		String dat=null;
		try{
			System.out.println("LOAD " + fileName);
			InputStreamReader inp = new InputStreamReader(
										Connector.openInputStream(fileName) );
			int i = 0;
			StringBuffer buf = new StringBuffer(512);
			while(i >= 0 && (char)i != '\t'){
				buf.append((char)i);
				i = inp.read();
			}
			inp.close();

			dat = buf.toString();
		}catch(Exception e){
			System.out.println("Exeption?:"+ fileName + "\n" + e.toString());
		}
		return dat;
	}


	public IGameState draw(Graphics g){
		if(!ready)return this;
		g.lock();
		cls(g);

		vg.setGraphics(g);
			stage.drawingEntry(vg);
		vg.draw();


		g.unlock(false);
		return this;
	}
	private void cls(Graphics g){
		g.setColor(g.getColorOfRGB(0x33,0x0,0x00));
		g.fillRect(0,0,inf.getScreenWidth(),inf.getScreenHeight());
	}
	
	private int findNextReplay(){
		int n = stageCnt+1;
		int l = SystemConfig.lastStage + 1;
		for(int i=0;i<l;i++){
			int current=(n + i) % (l + 1);
			if(SystemConfig.checkReplay( current ) >0)return current;
		}
		return stageCnt;
	}
	
	
	public boolean onKeyPress(int softKey){
		if(!ready)return true;
		switch (softKey) {
			case Display.KEY_UP:
//			case Display.KEY_2:
				vGamePad.setY(-1);
				break;
			case Display.KEY_DOWN:
//			case Display.KEY_8:
				vGamePad.setY( 1);
				break;
			case Display.KEY_LEFT:
//			case Display.KEY_4:
				vGamePad.setX(-1);
				break;
			case Display.KEY_RIGHT:
//			case Display.KEY_6:
				vGamePad.setX( 1);
				break;

			case Display.KEY_ASTERISK:
				giveupFlag=true;
				break;

			case Display.KEY_SOFT1:
				if(SystemConfig.softKeyEnable <0)break;
			case Display.KEY_0:
			case Display.KEY_1:
//			case Display.KEY_7:
				vGamePad.setButton(0,1);
				break;
			case Display.KEY_SOFT2:
				if(SystemConfig.softKeyEnable <0)break;
			case Display.KEY_3:
//			case Display.KEY_9:
			case Display.KEY_POUND:
				vGamePad.setButton(1,1);
				break;


			case Display.KEY_7:
				zoomFlag = 1;
				break;
			case Display.KEY_8:
			case Display.KEY_SELECT:
				zoomFlag = 0;
				stage.zoomReset();
				break;
			case Display.KEY_9:
				zoomFlag = -1;
				break;


			case Display.KEY_5:
				pauseFlag = true;
				break;

			default:
			return false;
		}
		return true;
	}
	public boolean onKeyRelease(int softKey){
		if(!ready)return true;
		switch (softKey) {
			case Display.KEY_UP:
//			case Display.KEY_2:
				vGamePad.setY(0);
				break;
			case Display.KEY_DOWN:
//			case Display.KEY_8:
				vGamePad.setY(0);
				break;
			case Display.KEY_LEFT:
//			case Display.KEY_4:
				vGamePad.setX(0);
				break;
 			case Display.KEY_RIGHT:
//			case Display.KEY_6:
				vGamePad.setX(0);
				break;

			case Display.KEY_ASTERISK:
				giveupFlag = false;
				giveupCnt=0;
				break;


			case Display.KEY_SOFT1:
				if(SystemConfig.softKeyEnable <0)break;
//			case Display.KEY_7:
			case Display.KEY_0:
			case Display.KEY_1:
				vGamePad.setButton(0,0);
				break;

			case Display.KEY_SOFT2:
				if(SystemConfig.softKeyEnable <0)break;
//			case Display.KEY_9:
			case Display.KEY_3:
			case Display.KEY_POUND:
				vGamePad.setButton(1,0);
				break;

			case Display.KEY_7:
			case Display.KEY_SELECT:
			case Display.KEY_8:
			case Display.KEY_9:
				zoomFlag = 0;
				break;


			default:
			return false;
		}
		return true;
	}


}

class StageStartState implements IGameState{
	IGameState parentState;
	GameStage stage;
	VGraphics vg;
	CanvasInfo inf;
	int cnt;

	public IGameState call(){
		cnt=50;
		for(int i=0;i<50;i++)stage.zoomIn();
		return this;
	}

	public IGameState action(){
		cnt--;
		stage.zoomOut();
		if(cnt>0)return this;
		else{
			stage.zoomReset();
			return parentState;
		}
	}
	public StageStartState(IGameState st,GameStage stg,VGraphics g,CanvasInfo ci){
		parentState = st;
		stage       = stg;
		vg          = g;
		inf         = ci;
	}


	public IGameState draw(Graphics g){
		g.lock();
		g.setColor(g.getColorOfRGB(0x33,0x0,0x00));
		g.fillRect(0,0,inf.getScreenWidth(),inf.getScreenHeight());
		vg.setGraphics(g);

			stage.drawingEntry(vg);

		vg.draw();
		g.unlock(false);
		return this;
	}

	public boolean onKeyPress(int softKey){
		return true;
	}
	public boolean onKeyRelease(int softKey){
		return true;
	}

}

class PlayerDyingState implements IGameState{
	IGameState parentState;
	GameStage stage;
	VGraphics vg;
	CanvasInfo inf;
	Player player;
	int cnt;

	public IGameState call(){
		player=stage.getPlayer();
		cnt=100;
		return this;
	}

	public IGameState action(){
		cnt--;
		if(cnt < 90)player.action();
		stage.zoomIn();
		stage.zoomIn();

		if(cnt>0)return this;
		else{
			return parentState.call();
//return this;
		}
	}
	public PlayerDyingState(IGameState st,GameStage stg,VGraphics g,CanvasInfo ci){
		parentState = st;
		stage       = stg;
		vg          = g;
		inf         = ci;
	}


	public IGameState draw(Graphics g){
		g.lock();
		g.setColor(g.getColorOfRGB(0x99,0x0,0x00,0x40));
		g.fillRect(0,0,inf.getScreenWidth(),inf.getScreenHeight());
		vg.setGraphics(g);
			stage.drawingEntry(vg);
		vg.draw();
		g.unlock(false);
		return this;
	}

	public boolean onKeyPress(int softKey){
		return true;
	}
	public boolean onKeyRelease(int softKey){
		return true;
	}

}


class StageClearState implements IGameState{
	IGameState parentState;
	GameStage stage;
	VGraphics vg;
	CanvasInfo inf;
	Player player;
	int cnt=80;


	public IGameState call(){
		player=stage.getPlayer();
		cnt=80;
		return this;
	}

	public IGameState action(){
		cnt--;
		if(player.getY() > -Blocks.halfsize)player.action();
		stage.zoomIn();

		if(cnt>0)return this;
		else{
			return parentState.call();
		}
	}
	public StageClearState(IGameState st,GameStage stg,VGraphics g,CanvasInfo ci){
		parentState = st;
		stage       = stg;
		vg          = g;
		inf         = ci;
	}


	public IGameState draw(Graphics g){
		g.lock();
		g.setColor(g.getColorOfRGB(0x00,0x0,0x66,0x40));
		g.fillRect(0,0,inf.getScreenWidth(),inf.getScreenHeight());
		vg.setGraphics(g);
			stage.drawingEntry(vg);
		vg.draw();
		g.unlock(false);
		return this;
	}

	public boolean onKeyPress(int softKey){
		return true;
	}
	public boolean onKeyRelease(int softKey){
		return true;
	}

}



class PauseMenuState implements IGameState{
	IGameState parentState;
	GameStage stage;
	VGraphics vg;
	CanvasInfo inf;
	Player player;
	int bestTime=0;
	Font f=Font.getDefaultFont();
	boolean retFlag = false;


	public int stageCnt;
	public boolean chgFlag = false;
	public boolean replayFlag = false;

	private boolean exitFlag = false;
	int sw,sh;
	int ww,hh;
	int px,py;
	int mx=1,my=1;

	public IGameState call(){
		stageCnt = ((GameState)parentState).getStageCnt();
		chgFlag = false;
		replayFlag = false;
		retFlag = false;
		frameCnt=0;
		bestTime=0;
		
		sw = inf.getScreenWidth();
		sh = inf.getScreenHeight();
		ww = (stage.getCols()-1)*Blocks.size;
		hh = (stage.getRows()-1)*Blocks.size;
		player = stage.getPlayer();
		px = player.getX();
		py = player.getY();
		inf.setSoftLabel(Canvas.SOFT_KEY_2,"I");
		rpCheck();



		return this;
	}

	
	int frameCnt;
	public IGameState action(){
		frameCnt++;
		if(frameCnt < 50 )stage.zoomIn();
		{
			px += mx;
			 if(px<= Blocks.size || px >= ww)mx*=-1;
			py += my;
			 if(py<= Blocks.size || py >= hh)my*=-1;
			 stage.setScrollPoint(px,py);
		}
		if(exitFlag)return null;
		if(retFlag){
			SystemConfig.save();
			if(!chgFlag)stage.zoomReset();
			inf.setSoftLabel(Canvas.SOFT_KEY_2,null);
			return parentState.call();
		}	
		return this;
	}
	public PauseMenuState(IGameState st,GameStage stg,VGraphics g,CanvasInfo ci){
		parentState = st;
		stage       = stg;
		vg          = g;
		inf         = ci;

	}


	public IGameState draw(Graphics g){
		g.lock();

		g.setColor(g.getColorOfRGB(0x33,0x0,0x00));
		g.fillRect(0, 0,sw,sh );

		vg.setGraphics(g);
		stage.drawingEntry(vg);
		vg.draw();
		int alpha = frameCnt *4;
		if(alpha>200)alpha=200;



		g.setColor(g.getColorOfRGB(0x00,0x0,0x00,alpha));
		g.fillRect(0, sh>>2,sw,sh >> 1 );

		String str = "Xe[W " + (stageCnt+1)  + "      (㉺ŕύX)";
		String tm=null;

		if(frameCnt %20 <10){
			g.setColor(g.getColorOfRGB(0xFF,0x00,0x00));
			centerText("*PAUSE*", sh * 1 / 3 ,g);
		}

		g.setColor(g.getColorOfRGB(0xFF,0xFF,0xFF));
		centerText(str, sh * 2 / 3 -60,g);

		if(bestTime>0)
			tm = "L^ " + bestTime + 
					      "         ([0]ōĐ)";
		else
			tm = "L^ ----                    ";
		centerText(tm, sh * 2 / 3  -40 ,g);
			//        "Xe[W 00      (㉺ŕύX)";
			//        "L^ xxxx         ([0]ōĐ)";
			//        "L^ ----                    ";

		String skey = "MENUL[: g    ([MENU]ŕύX)";
		g.setColor(g.getColorOfRGB(0xcc,0xcc,0xFF));
		if(SystemConfig.softKeyEnable <0)
			   skey = "MENUL[:gȂ ([MENU]ŕύX)";

		centerText(skey , sh * 2 / 3 ,g);



		g.unlock(false);
		return this;
	}

	private void centerText(String str , int yy , Graphics g){
		int strWidth = f.stringWidth(str);
		int strHeight = f.getHeight();
		g.drawString( str , (sw-strWidth) >> 1 , yy +  (strHeight>>1) );
	}

	public boolean onKeyPress(int softKey){
		int ls = SystemConfig.lastStage +1;
		switch (softKey) {
			case Display.KEY_UP:
				stageCnt =
					(stageCnt + 1) % ls;
				System.out.println("stageCnt="+stageCnt + " " + SystemConfig.lastStage );
				rpCheck();
				break;
			case Display.KEY_DOWN:
				stageCnt = 
					(stageCnt + (ls -1 )) % ls;
				rpCheck();
				break;


			case Display.KEY_POUND:
			case Display.KEY_SELECT:
				chgFlag = true;
				retFlag = true;
				break;

			case Display.KEY_0:
				rpCheck();
				if(bestTime>0){
					replayFlag = true;
					retFlag = true;
					chgFlag = true;
				}
				break;

			case Display.KEY_5:
				retFlag = true;
				chgFlag = false;
				break;

			case Display.KEY_SOFT1:
				SystemConfig.softKeyEnable 
					 = (SystemConfig.softKeyEnable > 0) ? -1:1;
				break;
			case Display.KEY_SOFT2:
				exitFlag = true;
				break;
/*
			case Display.KEY_LEFT:
			case Display.KEY_RIGHT:
			case Display.KEY_0:
			case Display.KEY_1:
			case Display.KEY_3:
			case Display.KEY_4:
			case Display.KEY_6:
			case Display.KEY_7:
			case Display.KEY_8:
			case Display.KEY_9:
				break;
*/

			default:
			return false;
		}
		return true;
	}
	public boolean onKeyRelease(int softKey){return true;}

	private void rpCheck(){
		if(stageCnt < 0)stageCnt=0;
		bestTime = SystemConfig.checkReplay(stageCnt);
	}

}