package characters;
import com.nttdocomo.ui.*;
import drawing.*;

public class CharacterGroup implements IDrawingEntry{
	public int getX(){return 0;}
	public int getY(){return 0;}
	public int getZ(){return 0;}
	
	private ICharacter[] vc;
	private int ptr=0;

	public CharacterGroup(int max){
		vc = new ICharacter[max];
	}
	
	public ICharacter getMember(int num){
		if(num >= vc.length)return null;
		return vc[num];
	}

	public ICharacter findIdle(){
		for(int i=0;i<ptr;i++){
			if(!vc[i].isAlive())return vc[i];
		}
		return null;
	}

	public boolean add(ICharacter cc)
	{
		if(ptr >= vc.length)return false;
		vc[ptr] = cc;
		ptr++;
		return true;
	}
	public int add(ICharacter[] cc)
	{
		int cnt=0;
		for(int i=cc.length-1;i>=0;i--){if(add(cc[i]))cnt++;}
		return cnt;
	}

	public boolean remove(ICharacter cc)
	{
		for(int i=vc.length-1;i>=0;i--){
			if(vc[i] == cc){
				vc[i] = vc[ptr-1];
				vc[ptr-1]=null;
				ptr--;
				return true;
			}
		}
		return false;
	}
	public int remove(ICharacter[] cc)
	{
		int cnt=0;
		for(int i=cc.length-1;i>=0;i--){if(remove(cc[i]))cnt++;}
		return cnt;
	}

	public boolean clear()
	{
		for( int i = ptr -1 ; i >= 0 ; i-- ){
			vc[i].kill();
			vc[i] = null;
		}
		ptr=0;
		return true;
	}

	public boolean drawingEntry(VGraphics g){
		for( int i = ptr - 1 ; i >= 0 ; i-- ){
			ICharacter cc = vc[i];
			if (cc.isAlive()){
				cc.drawingEntry(g);
			}
		}
		return true;
	}

	public boolean draw(VGraphics g){
		for( int i = ptr - 1 ; i >= 0 ; i-- ){
			ICharacter cc = vc[i];
			if (cc.isAlive()){
				cc.drawingEntry(g);
			}
		}
		return true;
	}


	public boolean action()
	{
		for(int i=ptr-1;i>=0;i--){
			ICharacter cc = vc[i];
			if (cc.isAlive())
				cc.action();
//			else
//				remove(cc);
		}
		return true;
	}

	public void reset()
	{
		for(int i=ptr-1;i>=0;i--)vc[i].reset();
	}

	public void kill()
	{
		for(int i=ptr-1;i>=0;i--)vc[i].kill();
	}

	public boolean collision(ICharacter m)
	{
		boolean ret = false;
		for( int i = ptr -1 ; i >= 0 ; i-- ){
			ICharacter cc = vc[i];
			if ( cc == m || !cc.isAlive() )	continue;
			 if(cc.collision(m)){
				if(cc.onCollided(m)){
					m.onCollided(cc);
				}
				ret = true;
			 }
		}
		return ret;
	}

	public void collision(CharacterGroup g)
	{
		for(int i = g.ptr-1 ; i>= 0 ; i--){
			collision(g.vc[i]);
		}
		
	}

}