package gaming;
import com.nttdocomo.ui.*;
import com.nttdocomo.io.*;
import drawing.*;
import characters.*;
import input.*;
import gaming.*;

public class GameStage implements IDrawingEntry{
	public static final int MODE_INIT   = 0;
	public static final int MODE_GAME   = 1;
	public static final int MODE_CLEAR  = 2;
	public static final int MODE_DEAD   = 3;
	public static final int MODE_GIVEUP = 4;
	public static final int MODE_REPLAY = 5;
	public static final int MODE_GAMEOVER=6;
	protected int mode = MODE_INIT;
	public int getMode(){return mode;}
	
	int cols = 15;
	int rows = 15;
	int goldCnt = 0;
	int[][] dat;
	private StringBuffer datStr;

	int[] colors;
	Image[] images;

	ScaledImage[] imgTable;
	CanvasInfo canvasInfo;
	
	protected Player player;
	public Player getPlayer(){return player;}
	protected CharacterGroup enemies,diggers,others;
	private Randomizer random;
	public int rnd(){return random.get();}

	
	int frameCnt=0;
	 public int getFrameCnt(){return frameCnt;}
	InputRecorder inputRecorder = new InputRecorder();
	boolean replayMode = false;
	boolean pauseFlag = false;

	int giveupFlag = -1;
	public void setGiveupFlag(){
		if(mode == MODE_GAME)giveupFlag = frameCnt+1;
	}
	
	String title= "";
	public String getTitle(){return title;}
	public void setTitle(String s){title=s;}

	int killCnt = 0;
	int diggerCnt = 0;
	boolean stageClear = false;
	int key = 0;

	IGamePad vGamePad;
	int startX = 0;
	int startY = 0;

	int width  = 0;
	int height = 0;
	int vWidth  = 0;
	int vHeight = 0;
	int scrollTop = 0;
	int scrollLeft = 0;
	Font font = Font.getDefaultFont();
	public int getScrollTop(){return scrollTop;}
	public int getScrollLeft(){return scrollLeft;}

	public int getRows(){return rows;}
	public int getCols(){return cols;}
	
	public int getX(){return 0;}
	public int getY(){return 0;}
	public int getZ(){return 0;}
	public boolean drawingEntry(VGraphics g){
		g.entry(this);
		player.drawingEntry(g);
		enemies.drawingEntry(g);
		diggers.drawingEntry(g);
		others.drawingEntry(g);
		return true;
	}
	public boolean draw(VGraphics g){
		int vw = canvasInfo.getVirtualWidth();
		int vh = canvasInfo.getVirtualHeight();

		int sx = scrollLeft >> Blocks.shiftsize;
		int sy = scrollTop  >> Blocks.shiftsize;

		int dw = vw / Blocks.size + sx +2;
		int dh = vh / Blocks.size + sy +2;
		
		if(dw > cols)dw = cols;
		if(dh > rows)dh = rows;
		if(sx < 0) sx=0;
		if(sy < 0) sy=0;

		for(int y=sy;y<dh;y++){
			for(int x=sx; x<dw;x++){


			if(Blocks.isDigger(dat[y][x]))continue;
			int block = (dat[y][x] & ~(Blocks.maskbits));
			if( block !=0 && block != Blocks.hladder)
				g.drawImage(
					images[block],
					x*Blocks.size -scrollLeft,y*Blocks.size - scrollTop,
					Blocks.size,Blocks.size 
				);
			}
			
		}
		
		String gs = title + "  GOLD " + goldCnt + "  " + "TIME " + frameCnt;
		int gw = font.stringWidth(gs);
		g.setColor(0xffff00);
		g.drawString(gs  , 0 , vh + scrollTop);
		
		return true;

	}
	
	public GameStage(CanvasInfo ci){
		canvasInfo = ci;
		initImage();
		random = new Randomizer();
		enemies = new CharacterGroup(32);
		diggers = new CharacterGroup(64);
		others  = new CharacterGroup(32);
		inputRecorder = new InputRecorder();
		datStr = new StringBuffer();
	}

	protected boolean initImage(){
		colors = Blocks.getColors();
		images = Blocks.getImages();

		return true;
	}



	public boolean action(){
		frameCnt++;
		int padState;
//		if(inputRecorder.isHasNext()){
		if(mode==MODE_REPLAY){
			padState = inputRecorder.play(vGamePad);
			if(!inputRecorder.isHasNext())mode = MODE_GAME;
		}else{
//			if(mode ==MODE_REPLAY)mode = MODE_PLAY;
			padState = //vGamePad.getPadStatus();//
					inputRecorder.record(vGamePad);
		}
		player.setPadStatus(padState);

		diggers.action();
		player.action();
		enemies.action();
		 diggers.collision(enemies);
		 diggers.collision(player);
		 enemies.collision(player);

		others.action();

		setScrollPoint();
/*
		scoreBoard.action();
		inputViewer.action();
*/
		if(frameCnt == giveupFlag){
			player.giveup();
		}

		if(player.isDead()){
			mode = MODE_DEAD;
//			System.out.println(getMapData());
			return false;
		}else
		if(goldCnt==0 && player.getY() < Blocks.size){
			mode = MODE_CLEAR;
//			System.out.println(getMapData());
			return false;
		}
		return true;
	}

	protected void reset(){
		mode = MODE_INIT;
		cols = 15;
		rows = 15;
		dat = null;
		datStr.setLength(0);
		goldCnt = 0;
		frameCnt = 0;
		giveupFlag = -1;

		enemies.reset();
		diggers.reset();
		others.reset();
		
		inputRecorder.reset();

		SystemConfig.resetSysVer();

		System.gc();
	}

	protected String[] split(String text,String cut){
		String[] ret;
		String tmp = text;
		int p = 0,cnt=0;
		while(true){
			p = tmp.indexOf(cut,p+1);
			if(p < 0)break;
			cnt++;
		}
		ret = new String[cnt];
		p=0;cnt=0;
		while(true){
			p = tmp.indexOf(cut);
			if(p < 0)break;
			ret[cnt] = tmp.substring(0,p);
			tmp = tmp.substring(p+1);
			cnt++;
		}
		return ret;
	}

	public boolean loadStage(String text){
		reset();
//		System.out.print("TEXT = \n###\n" + text + "\n###\n");
		String[] tmp = split(text.trim() + "\n","\n");
		dat = new int[tmp.length + 10][];
		int rndSeed = -1;//

		StringBuffer rpCode = new StringBuffer();
		boolean rpFlag = false;

		rows=0;
		for(int i=0 ; i < tmp.length; i++){
			String ln = tmp[i];
//			System.out.println("!!! " + ln);

			int eq = ln.indexOf("=");

			if(ln.indexOf("[/replay]") >= 0){
				rpFlag = false;
			}else
			if(rpFlag){
				rpCode.append(ln.trim());
//				rpCode.append('\n');
			}
			else
			if(ln.indexOf("[replay]") >= 0){
				rpFlag = true;
			}
			else
			if(eq < 0 && ln.length() > 3){
				datStr.append(ln);
				datStr.append('\n');
				if(i==0) dat[0] = new int[ln.length()+10];
				rows++;
				dat[i+1] = new int[ln.length()+2];
				dat[i+1][0] = 2;
				dat[i+1][ln.length()+1] = 2;
				for(int j=0;j<ln.length();j++){
					char c = tmp[i].charAt(j);
					if(c==0)c='0';//continue;
//					System.out.print(":"+c + " " + ((int)c));
//					System.out.print(":"+ (((int)c) - ((int)'0')));
					   dat[i+1][j+1] = ((int)c) - ((int)'0');
					   if(dat[i+1][j+1]<0)dat[i+1][j+1]=0;
					if(dat[i+1][j+1] == 7)goldCnt++;
				}
			}
			else
			if(ln.length() > 3){
				String nm = ln.substring(0,eq).trim();
				String vl = ln.substring(eq +1).trim();
				System.out.println("param:"+ nm + "=" + vl);
				if(nm.equals("randomizeValue")){
					rndSeed = Integer.parseInt(vl,10);
				}else
				if(nm.equals("giveupFlag")){
					giveupFlag = Integer.parseInt(vl,10);
				}else
				if(nm.equals("sysVer")){
					SystemConfig.currentSysVer = Integer.parseInt(vl,10);
				}
			}
//			System.out.println(" " + ln +" cols=" + ln.length() + " rows="+ rows);
		}
		if(rndSeed<0)rndSeed = (new java.util.Random()).nextInt();

		cols = dat[1].length;
		rows = rows+2;

		dat[rows-1] = new int[cols];
		for(int i=0;i<cols;i++){
			dat[0][i] = 2;
			dat[rows-1][i] = 2;
		}

/*
		for(int i=0;i<rows;i++){
				System.out.print("[");
			for(int j=0;j<cols;j++){
				System.out.print("," + dat[i][j]);
			}
				System.out.println("]" + i);
		}
//*/
		width = cols*Blocks.size;vWidth=width;
		height= rows*Blocks.size;vHeight=height;
		canvasInfo.setVirtualScreenSize(width,height);
		float fx = canvasInfo.getScaleX();
		float fy = canvasInfo.getScaleY();
		float fxx;
		if( fx < fy )vHeight =(int) ( height * (fy/fx)+.5) ;
		else		 vWidth  =(int) ( width  * (fx/fy)+.5);
		canvasInfo.setVirtualScreenSize(vWidth,vHeight);


		frameCnt = 0;
		random.set( rndSeed );
		createCharacters();
		
		if(rpCode.length() > 0){
			mode = MODE_REPLAY;
			inputRecorder.setRecordData(rpCode.toString());
//			System.out.println("rpCode:\n\n"+rpCode.toString());
//			System.out.println("decompress:\n\n"+inputRecorder.getRecordData());
		}else{
			mode = MODE_GAME;
		}

		return true;
	}

	protected boolean createCharacters(){
		if(player == null)player = new Player(this,canvasInfo);
		int enemyCnt=0;
		enemies.kill();
		diggers.kill();
		others.kill();

		for(int y=0;y<rows;y++){
			for(int x=0; x<cols;x++){
				int b = dat[y][x];
				switch(b){
					case 8:	// Enemy
						Enemy e = (Enemy)enemies.getMember(enemyCnt);
						if(e ==null){
							e = new Enemy(this,canvasInfo);
							enemies.add(e);
						}
						e.reset();
						e.setTarget(player);
						e.moveTo(x * Blocks.size , y * Blocks.size);
						enemyCnt++;
						setEnemyMask(x,y);
					break;
					case 9: // Player
						player.reset();
						player.moveTo(x*Blocks.size,y*Blocks.size);
						dat[y][x] = Blocks.blank;
					break;
					case 10: // Goal
						dat[y][x] = Blocks.blank;
					break;
					case 11: // MBlock
						dat[y][x] = Blocks.blank;
					break;
					case 12: // HEnemy
						dat[y][x] = Blocks.brick;
					break;
				}
			}
		}
		if(enemyCnt>0)Enemy.initActionTable(enemyCnt);
		return true;
	}

	public int getXYBlock(int xx,int yy){
		 if(xx <= 0 || xx >= cols-1 || yy >= rows-1)return Blocks.rock;
		 if(yy < 0)return Blocks.blank;
		 return dat[yy][xx];
	}
	
	public int getXYPointBlock(int xx , int yy){
		return getXYBlock(xx >> Blocks.shiftsize , yy >> Blocks.shiftsize );
	}

//*
	public int getXYBlockR(int xx,int yy){
		 if(xx <= 0 || xx >= cols-1 || yy <= 0)return Blocks.blank;
		 if(yy >= rows-1)return Blocks.rock;
		 return dat[yy][xx];
	}
	public int getXYBlockR2(int xx,int yy){
		 if(xx <= 0 || xx >= cols-1 || yy >= rows-1)return Blocks.rock;
		 if(yy < 0)return Blocks.blank;
		 return dat[yy][xx];
	}
//*/
	public void setXYBlock(int block,int xx,int yy){
		if(xx<1||yy<0||xx>=cols-1 || yy>=rows-1)return;
		dat[yy][xx] = (dat[yy][xx] & Blocks.maskbits) + block;
	}


	public void setEnemyMask(int xx,int yy){
		if(xx<1||yy<0||xx>=cols-1 || yy>=rows-1)return;
		dat[yy][xx]  |= Blocks.enemybit;
	}

	public void resetEnemyMask(int xx,int yy){
		if(xx<1||yy<0||xx>=cols-1 || yy>=rows-1)return;
		dat[yy][xx]  &= ~Blocks.enemybit;
	}


	public void setDiggerMask(int xx,int yy){
		if(xx<1||yy<0||xx>=cols-1 || yy>=rows-1)return;
		dat[yy][xx]  |= Blocks.diggerbit;
	}

	public void resetDiggerMask(int xx,int yy){
		if(xx<1||yy<0||xx>=cols-1 || yy>=rows-1)return;
		dat[yy][xx]  &= ~Blocks.diggerbit;
	}


	public int getGold(int xx , int yy,boolean flag){
		if(xx<1||yy<0||xx>=cols-1 || yy>=rows-1)return 0;
		if(flag){
			goldCnt--;
			if(goldCnt==0)createLadder();
		}
		setXYBlock(0,xx,yy);
		return 1;
	}
	public boolean setGold(int xx , int yy ){
		if(xx<1||yy<0||xx>=cols-1 || yy>=rows-1)return false;
		int b = getXYBlock(xx,yy);
		if(Blocks.isBlank(b)){
			setXYBlock(Blocks.gold,xx,yy);
		}
		return false;
	}
	public void loseGold(int xx , int yy){
		goldCnt--;
		if(goldCnt==0)createLadder();
	}

	public void setGamePad(IGamePad g){
		vGamePad = g;
	}

	

	public void createLadder(){
		 for(int y = 0 ; y < rows - 1; y++ ){
			for(int x = 1 ; x < cols-1 ; x++ ){
				if( Blocks.isHLadder(dat[y][x]))
							setXYBlock( 3 , x , y  );
				if( Blocks.isLadder(dat[y][x]) && y<=2){
							setXYBlock( 3 , x , 0  );
							setXYBlock( 3 , x , 1  );
				}
			}
		 }
	}

	public ICharacter createDigger(ICharacter mst,int bx , int by){
		Digger dig = (Digger)diggers.findIdle();
		if(dig == null){
			dig = new Digger(this,canvasInfo);
			diggers.add(dig);
		}
		dig.call(bx,by,this,canvasInfo,mst);
		return dig;
	}

	public void zoomIn(){
		int vw = canvasInfo.getVirtualWidth();
		int vh = canvasInfo.getVirtualHeight();
		vw -= vw>>4;//width  / 16;
		vh -= vh>>4;//height / 16;
		if( vw < (Blocks.size*3) || ( vh < (Blocks.size*3) ) ){
//			vw =vWidth  >>4 ; vh=vHeight >>4 ;
			return;//vh = height >>4;
		}
		canvasInfo.setVirtualScreenSize(vw,vh);
		setScrollPoint();
	}
	public void zoomReset(){
		canvasInfo.setVirtualScreenSize(vWidth,vHeight);
		setScrollPoint();
	}
	public void zoomOut(){
		int vw = canvasInfo.getVirtualWidth();
		int vh = canvasInfo.getVirtualHeight();
		vw += vw>>4;
		vh += vh>>4;
		if(vw > width && vh > height)zoomReset();
		else
		canvasInfo.setVirtualScreenSize(vw,vh);
		setScrollPoint();
	}

	public void setScrollPoint(){
		int vw = canvasInfo.getVirtualWidth();
		int vh = canvasInfo.getVirtualHeight();
			int pcx = player.getX() + Blocks.halfsize;
			int pcy = player.getY() + Blocks.halfsize;
		setScrollPoint(pcx - (vw>>1) , pcy - (vh>>1));
	}
	public void setScrollPoint( int sx,int sy ){
		int vw = canvasInfo.getVirtualWidth();
		int vh = canvasInfo.getVirtualHeight();

		if(vw < width){
			scrollLeft = sx;
			if(scrollLeft + vw > width)scrollLeft = width - vw;
			if(scrollLeft < 0 )scrollLeft=0;
		}else{
			scrollLeft = ((width - vw)/2);
		}

		if(vh < height){
			scrollTop  = sy;
			if(scrollTop  + vh > height)scrollTop = height - vh;
			if(scrollTop < 0)scrollTop = 0;
		}else{
			scrollTop = ((height - vh)/2);
		}

	}


	public String getMapData(){
//		StringBuffer buf = new StringBuffer();
//		buf.append(datStr);
		return datStr.toString();
	}
	public String getReplayCode(){
		StringBuffer buf = new StringBuffer();

		buf.append("randomizeValue=" + random.seed + "\n");
		buf.append("sysVer=" + SystemConfig.currentSysVer + "\n");
		if(giveupFlag>=0)
			buf.append("giveupFlag=" + giveupFlag + "\n");
		if(inputRecorder.getLength() >0){
			buf.append("\n[replay]\n");
			buf.append(inputRecorder.getRecordData());
			buf.append("\n[/replay]\n");
		}
		String ret = buf.toString();
		System.out.println(ret);
		return ret;
	}

}

class Randomizer{
	public int seed = 0;
	private int rnd = 0;
	private int rndA=8*5+5;
	private int rndC=1213;
	private int rndM=0x10000;
	public int get(){
		rnd = ( rndA * rnd + rndC ) % rndM;
//		System.out.println("rnd=" +rnd);
		return  rnd ;
	}
	public void set(int s){rnd = Math.abs(s % 0x1000);seed=rnd;}
} //
